					PROCESSOR		17C42

; This is the basic outline for a program that generates a
; variable PWM output. The PWM's period and duty cycle can
; be varied. The new period (NEW_PR1) and the new duty cycle
; (NEW_DC1 and NEW_DC1Q) are loaded by the user program.
; The peripheral interrupt routine loads the new period value
; (frequency) into the PR1 register. A subroutine (PWM_UD)
; is also used to ensure that the 10-bit duty cycle registers
; are updated in the same PWM cycle, i.e. the timer match does not
; occur between two duty cycle register writes.
;
; The duty cycle value gets latched on the overflow (Period match)
; of the timer. The period value gets modified as soon as the period
; register is changed. Therefore care must be taken in updating 
; the period register. In cases where the period value is modified
; to a smaller value, we must ensure that the Timer counter is less
; then this value when the period register is updated (TMR1 < new PR1).
; If TMR1 is greater then PR1, the counter will count to FFh, rollover
; to 00H, and only cause the overflow interrupt when it then reaches
; the period value. This would give a wrong PWM output.
;
; In this example the event which cause the PWM to be updated 
; is an asynchronous event. A low frequency signal was placed on
; port pin RB0.
; For a high level the PWM registers are updated as follows:
;    PR1 = 7Fh, PW1DCH = 3Fh, and PW1DCL = 40h
; For a low level the PWM registers are updated as follows:
;    PR1 = 1Fh, PW1DCH = 07h, and PW1DCL = 80h
;
; Do the EQUate table
;
NEW_DC1		EQU	0x20		; New PWM1 duty cycle value
NEW_DC1Q	EQU	0x21		; 
NEW_PR1		EQU	0x22		; New PWM1 period value
PWM_WIN		EQU 0x25		; Register for the PWM window cycle count
CALC_PR		EQU	0x26		; Calculated period value
FLAG_REG	EQU	0x27		; Register for flag bits
;
DC1H		EQU	0x1A		; PWM registers for high time
DC1QH		EQU	0x1B
PR1H		EQU	0x1C
;
DC1L		EQU	0x1D		; PWM registers for low time
DC1QL		EQU	0x1E
PR1L		EQU	0x1F
;
;
END_OF_PROG_MEM		EQU 0x07FF
;
ALUSTA		EQU	0x04
CPUSTA		EQU	0x06
INTSTA		EQU	0x07
W			EQU	0x0A
;
DDRB		EQU	0x11		; Bank 0
PORTB		EQU	0x12
;
PIR			EQU	0x16		; Bank 1
PIE			EQU	0x17
;
TMR1		EQU	0x10		; Bank 2
TMR2		EQU	0x11
TMR3l		EQU	0x12
TMR3h		EQU	0x13
PR1			EQU 0x14
PR2			EQU 0x15
PR3L		EQU 0x16
PR3h		EQU 0x17
;
PW1DCL		EQU	0x10		; Bank 3
PW2DCL		EQU	0x11
PW1DCH		EQU	0x12
PW2DCH		EQU	0x13
TCON1		EQU	0x16
TCON2		EQU	0x17
 PAGE
		ORG		0x0000			; Origin for the RESET vector
		GOTO	START			; On reset, go to the start of
								;   the program	
		ORG		0x0008			; Origin for the external RA0/INT
								;   interrupt vector
		GOTO	EXT_INT			; Goto the ext. interrupt
								;   on RA0/INT routine
		ORG		0x0010			; Origin for the RTCC
								;   overflow interrupt vector
		GOTO	RTCCINT			; Goto the RTCC overflow interrupt
								;   routine
		ORG		0x0018			; Origin for the external
								;   RA1/RT interrupt vector
		GOTO	RT_INT			; Goto the ext. interrupt on 
								;   RA1/RT routine
		ORG		0x0020			; Origin for the interrupt vector
								;   of any enabled peripheral
;
; The interrupt routine for any peripheral interrupt, This routine
; only deals with Timer1 interrupt.
;
; Time required to execute interrupt routine. Not including
; interrupt latency (time to enter into the interrupt routine)
;
; 	case1 - only T1 overflow             = 12 cycles
;	case2 - Other                        = Infinite Loop
;
;
PER_INT	MOVLB	1				; Select register Bank 1
		BTFSS	PIR,4			; Did Timer1 overflow?
ERROR	GOTO	ERROR			; Not a Timer1 overflow.
								;   No other interrupts should
								;   be enabled, so error.
;
; Once the enabled Timer1 overflow occurs, the period register
; is loaded. This PWM waveform will remain until the PWM duty
; cycle and / or period is updated. Until such update, there is no
; S/W overhead from T1 interrupts (T1 interrupts can be disabled).
;
; NOTE: If PW1DCH >= PR1, then the duty cycle of this PWM output
;       is 100%.
;
; NOTE: The new Period register (PR1) value, must always be greater
;       than the value in the Timer1 register (TMR1). If a PR1 value
;       is loaded that is less then the TMR1 value, the timer will 
;       continue to count until it reaches the PR1 value. I.E. TMR1
;       will overflow at FFh and the count to the new PR1 value.
;       Minimum PR1 value is 0Ah, due to time to load new values and
;       execute the peripheral interrupt service routine.
;
T1OVFL	BCF		PIR,4			; Clear Overflow interrupt flag
		MOVLB	2				; Bank2
		MOVFP	NEW_PR1,PR1		; Load this period value
		MOVLB	1				; Bank 0
		BCF		PIE, 4			; Disable T1 interrupt
								;   (until transition on PORTB0)
		MOVLB	0				; Bank 0
		BTG		PORTB, 7		; Transition PortB 7 pin (H->L, or L->H)
		RETFIE					; Return from Interrupt
 PAGE
;
; This is the start of the program.
;
START	BSF		CPUSTA,4		; Disable ALL interrupts via the
								;   Global Interrupt Disable 
								;   (GLINTD) bit.
								;
MAIN							; Place Main program here
		MOVLB	3				; Select register Bank 3
		CLRF	TCON2,0			; Stop the timers, Single Capture
		MOVLW	0x070			; Initalize TCON1 so that
		MOVWF	TCON1			;   T1 (8-bit), T2 (8-bit), 
								;   and T3 run off the internal
								;   system clock. Timer3 uses
								;   period register
		MOVLW	0x0D			; Load the PWM window cycle value
		MOVWF	PWM_WIN			;
;
		MOVLB	0				; Select register Bank 0
		SETF	DDRB, 1			; Port B is an input
		CLRF	PORTB, 1		; Set output values to 0 (for PORTB)
		BCF		DDRB, 7			; PORTB7 is an output. Used to triger a scope
		CLRF	FLAG_REG, 1		; Clear the Flag registers
;
; Load registers with the PWM values that we will switch between. One set
; for the time PORTB0 is high and another set for when low.
;
; For a high level the PWM registers are updated as follows:
;    PR1 = 7Fh, PW1DCH = 3Fh, and PW1DCL = 40h
;    At 16Mhz this gives a period of 31.75 us, and a duty cycle of 16.625 us
; For a low level the PWM registers are updated as follows:
;    PR1 = 1Fh, PW1DCH = 07h, and PW1DCL = 80h
;    At 16Mhz this gives a period of 7.75 us, and a duty cycle of 6.00 us
;
		MOVLB	3				; Bank 3		
		MOVLW	0x3F			; The Duty Cycle initial value is
		MOVPF	W, DC1H			;   50% of the initial period
		MOVLW	0x40			; 
		MOVPF	W, DC1QH		; Duty Cycle low = 01
		MOVLW	0x07			; The Duty Cycle initial value is
		MOVPF	W, DC1L			;   25% of the initial period
		MOVLW	0x80			; 
		MOVPF	W, DC1QL		; Duty Cycle low = 10
;
		MOVLB	2				; Bank 2
		MOVLW	0x7F			;
		MOVPF	W, PR1H			; The initial period value is 50%
								;   of full scale (for High)
		MOVLW	0x1F
		MOVPF	W, PR1L			; The initial period value is 12.5%
								;   of full scale (for Low)
;
;
; Default PWM values should be set, and the timer should be started
; and the interrupts enabled.
;
		MOVLW	0xF0			; Load the Period register
		MOVWF	PR1				;
		MOVLB	3				; Select register Bank 3
		MOVLW	0xC0			; Load the T1 duty cycle register
		MOVWF	PW1DCH			;
		MOVWF	PW1DCL			; effectively loaded with 0
		MOVLW	0x31			;** Enable PWM1 and PWM2 outputs
		MOVWF	TCON2			;**   and turn on Timer1.

		BSF		INTSTA,3		; Turn on Peripheral Interrupts
		MOVLB	1				; Select register Bank 1
		MOVLW	0x10			; Enable Timer1 overflow
		MOVWF	PIE				;   Interrupts (when GLINTD = 0)
		BCF		CPUSTA,4		; Enable ALL interrupts
		MOVLB	0				; Bank 0
 PAGE
;
; Only need to update PWM values on the first occurance of a new level on RB0,
; Else loop waiting for level to change. 
;
HIGH1ST	BCF		FLAG_REG, 0		; First time in loop (this cycle) = True
HIGHCYC	BTFSS	PORTB, 0		; Is PortB0 low
		GOTO	LOW1ST			; PORTB0 = L
		BTFSC	FLAG_REG, 0		; Is this the First High time (this cycle)?
		GOTO	HIGHCYC			; Loop looking for low signal on PortB0
		BSF		FLAG_REG, 0		; Set First time in loop (this cycle) = False
; 
; Here is where we update the PWM values (period and Duty cycle) for high level.
;
		MOVLB	3				; Bank 3
		MOVPF	DC1H, NEW_DC1	; 
		MOVPF	DC1QH, NEW_DC1Q	;
		MOVPF	PR1H, NEW_PR1	;
		CALL	PWM1_UD			;
		GOTO	HIGHCYC			; Loop looking for low signal on PortB0
;
;
LOW1ST	BCF		FLAG_REG, 0		; First time in loop (this cycle) = True
LOWCYC	BTFSC	PORTB, 0		; Is PortB0 high
		GOTO	HIGH1ST			; PORTB0 = H
		BTFSC	FLAG_REG, 0		; Is this the First Low time (this cycle)?
		GOTO	LOWCYC			; Loop looking for high signal on PortB0
		BSF		FLAG_REG, 0		; First time in loop (this cycle) = False
; 
; Here is where we update the PWM values (period and Duty cycle) for low level.
;
		MOVLB	3				; Bank 3
		MOVPF	DC1L, NEW_DC1	; 
		MOVPF	DC1QL, NEW_DC1Q	;
		MOVPF	PR1L, NEW_PR1	; 
		CALL	PWM1_UD			;
		GOTO	LOWCYC			; Loop looking for high signal on PortB0

 PAGE
;
; This code segment ensure that all PWM values (period and duty cycle)
; are updated at the same time. This is done by ensuring that the Timer
; is at least PWM_WIN (0Dh) cycles before the PR1 value (PR1 - PWM_WIN > TMR1).
; If not a "glitch" could occur in the PWM waveform. When only the 1st duty
; cycle register is latched for this PWM cycle, and the following PWM period
; will latch the 2nd duty cycle register.
;
PWM1_UD BSF		CPUSTA, 4		; Disable Global Interrupts
		MOVLB	2				; Bank 2
		MOVFP	TMR1, W			; Load W reg. with Timer1 value
		SUBWF	PR1, 0			; PR1 - TMR1 -> W reg.
		CPFSLT	PWM_WIN			; Check if Timer1 is about to overflow
		GOTO	PWM1_UD			; Overflow would have occurred during
								;   PWM updates, Delay a few cycles
		MOVLB	3				; Bank 3
		MOVFP	NEW_DC1, W		; Your New PWM MSB
		MOVWF	PW1DCH			; Loaded in duty cycle buffer
		MOVFP	NEW_DC1Q, W		; Your New PWM LSB
		MOVWF	PW1DCL			; Loaded in duty cycle buffer
		MOVLB	1				; Back to Bank 1
		BCF		PIR, 4			; Clear T1 Overflow interrupt flag
		BSF		PIE, 4			; Enable T1 int
		BCF		CPUSTA, 4		; Enable Global Interrupts
		MOVLB	0				; Bank 0
		RETURN					;** this does not need to be implemented
								;**  as a subroutine.
;
; Other Interrupt routines. (Not utilized in this example)
;
EXT_INT	RETFIE					; RA0/INT interrupt routine
								;   (NOT used in this program)
RTCCINT	RETFIE					; RTCC overflow interrupt routine
								;   (NOT used in this program)
RT_INT	RETFIE					; RA1/RT interrupt routine
								;   (NOT used in this program)
								;
SRESET	GOTO	START			; If program became lost, goto
								;   START and reinitalize.
;
;
; When the executed address is NOT in the program range, the
; 16-bit address should contain all 1's (a CALL 0x1FFF). At
; this location you could branch to a routine to recover or
; shut down from the invalid program execution.
;
		ORG		END_OF_PROG_MEM	; 
		GOTO	SRESET			; The program has lost it's mind,
								;   do a system reset
		END
