;************************************************************************
;                 	Dual Tone Generation
;
;	A generic resonator subroutine is implemented to generate
; tones. Samples Of A Sin/Cos Wave are generated using
; recurssive techniques. Table Lookups are thus avoided
; This is especially useful in generating multiple tones
; (e.g. DTMF tone generation, tone signalling, etc), or programmable 
; tone generation which may vary for each application unit.
;
;************************************************************************
;
	include "17c42.h"
;
	include "17c42.mac"
;
;*****************************************************************************
; 	TBLADDR
;
; DESCRIPTION:
; 	Load 16 bit table pointer with specified label
;
; TIMING (in cycles):
;	4
;

TBLADDR	MACRO	label

	MOVLW	(label)
	MOVWF	tblptrl
	MOVLW	page	(label)
	MOVWF	tblptrh

	ENDM

;*****************************************************************************
; 	ADDLBL
;
; DESCRIPTION:
; 	Add A Label (16 bit constant) To A File Register (16 bit)
;
; TIMING (in cycles):
;	4
;

ADDLBL	MACRO	label,f

	MOVLW	(label)
	ADDWF	f+B0
	MOVLW	page	(label)
	ADDWFC	f+B1

	ENDM

;*****************************************************************************

	LIST
;
	CBLOCK	0
		B0,B1,B2,B3	; RAM offset constants
	ENDC
;
	CBLOCK  0x18
		AARG,AARG1		; 16 bit multiplier A
		BARG,BARG1		; 16 bit multiplicand B
		DPX,DPX1,DPX2,DPX3      ; 32 bit multiplier result = A*B
	ENDC
;
	CBLOCK				; Generic Resonator Variables
		K1,K11				; K1
		SinCos_2C,SinCos_2,SinCos1_2	; y(n-2), Init Value = K3
		SinCos_1,SinCos1_1		; y(n-1), Init Value = K2
		SinCos,SinCos1			; y(n)
		
	ENDC

	CBLOCK
		tone1K1,tone1K11		; Tone 1 variables
		tone1_2C,tone1_2,tone11_2
		tone1_1,tone11_1
		tone1,tone11

		tone2K1,tone2K11		; Tone 2 variables
		tone2_2C,tone2_2,tone21_2
		tone2_1,tone21_1
		tone2,tone21

		dualTone,dualTone1		; tone1+tone2
	ENDC

;
;************************************************************
;
#define	ADD_OFFSET	FALSE
;
#define	coeff_addr	0x0030	; prog mem addr of sine wave frequency,etc
#define DummyAddr	0x0A00
;
;******************************************************************
;		Test Routine For Sin Wave Generation
;
;	(a) Call Intialization Subroutine to read desired sine wave
;		freq from prog mem to data mem.
;	(b) Call the resonator subroutine to generate samples of desired
;		sine wave.
;	(c) Perform a Dummy Table Write of The Sine Wave Data, so that
;		PIC-MASTER emulator can capture the data into trace buffer
;	
;******************************************************************
;
	ORG	0x0000
	goto	start
;
;******************************************************************
;		Resonator Co-efficients For Desired Tones

	ORG	coeff_addr
	
; Tone 1 Resonator Constants
; Sample Rate = fs = 8 khz, Tone Freq = f = 0.800 Khz
;
	DATA	26510		; K1 = COS(wT) = COS(360*f/fs)
	DATA	9630		; K2 = SIN(wT) = SIN(360*f/fs) = init value of y(n-1)
	DATA	0		; K3 is init value of y(n-2)
;
; Tone 2 Resonator Constants
; Sample Rate = fs = 8 khz, Tone Freq = f = 1.10 Khz
;
	DATA	21281		; K1 = COS(wT) = COS(360*f/fs)
	DATA	6229		; K2 = SIN(wT) = SIN(360*f/fs) = init value of y(n-1)
	DATA	0		; K3 is init value of y(n-2)
;
;******************************************************************
;
	ORG	0x0040
start
	call	Coeff_Read
; load table pointers with a dummy  addr for PIC-MASTER data capture
	MOVK16	DummyAddr,tblptrl
NextSample
	nop
	tlwt	0,dualTone
capture
	tablwt	1,0,dualTone+B1		; for PIC-MASTER trace capture
	nop
;
	movlw	tone1K1			; load indirect address pointer
	movwf	fsr0			; for Tone 1
	call	Resonator		; Compute next sample of tone 1

	movlw	tone2K1			; load indirect address pointer
	movwf	fsr0			; for Tone 2
	call	Resonator		; compute next sample of tone 2
;
; Compute Tone1 + Tone2 for dual tone generation
;
	ADD16ACC	tone1,tone2,dualTone	; dualTone = tone1 + tone2
;
	goto	NextSample
;
self	goto	self
;
;***********************************************************************
; Initialization routine :
; Read Tone 1 & Tone 2 Resonator Frequencies from Program Memory To
; Data Memory 
;
;	Program Memory	:	3 + 8*(#of tones to be generated)
;	Timing		:	4 + 11*(#of tones to be generated)
;***********************************************************************

Coeff_Read

	TBLADDR	coeff_addr
	tablrd	0,1,tone1K1
	tlrd	0,tone1K1
	tablrd	1,1,tone1K1+B1	; read K1
	tlrd	0,tone1_1
	tablrd	1,1,tone1_1+B1	; read K2
	tlrd	0,tone1_2
	tlrd	1,tone1_2+B1	; read K3
	clrf	tone1_2C

	tablrd	0,1,tone2K1
	tlrd	0,tone2K1
	tablrd	1,1,tone2K1+B1	; read K1
	tlrd	0,tone2_1
	tablrd	1,1,tone2_1+B1	; read K2
	tlrd	0,tone2_2
	tlrd	1,tone2_2+B1	; read K3
	clrf	tone2_2C

	return
;
;***********************************************************************
;			Resonator Subroutine
;
; Before calling this routine, load the indirect register, FSR0
; with the starting RAM address of the desired Tone Variables, 
; ( eg. For Tone1 Generation, load FSR0 with "Tone1K1" address.
;
; 
;
; Timing (worst case) :
;		20 + 36 + (worst case multiplier time)
;		= 56 + 179 = 235 cycles 
;		= 58.75 uS @ 16Mhz
;		= 47.00 uS @ 20Mhz
;		= 37.60 uS @ 25 Mhz
;
; Memory Requirements :
;		Program Memory	: 54 locations
;		Data Memory	: 9 + 9*(#of tones to be generated)
;
;***********************************************************************
;

Resonator
	
;
; Transfer tone varialbles to resonator's variables using indirect addressing
; This is necessary for making the "Resonator" a generic
; subroutine, so that the same code can be called for various
; tones.
;  Indirect addressing mode can be used throught the code in this 
; subroutine, but is less efficient.
;
	bcf	_fs1
	bsf	_fs0		; auto increment FSR0
	movpf	indf0,K1+B0
	movpf	indf0,K1+B1
	movpf	indf0,SinCos_2C
	movpf	indf0,SinCos_2+B0
	movpf	indf0,SinCos_2+B1
	movpf	indf0,SinCos_1+B0
	movpf	indf0,SinCos_1+B1
	movpf	indf0,SinCos+B0
	movpf	indf0,SinCos+B1

;
; Compute 2*K1*y(n-1)
;
	MOVFP16	K1,BARG		
	MOVFP16	SinCos_1,AARG
	call	DblMult			
	RLC32	DPX			; DPX = 2*K1*y(n-1)
;
; subtract y(n-2)*(2**15)
;
	clrf	SinCos_2C
	RRC24	SinCos_2C
	SUB24	SinCos_2C,DPX1		; DPX = 2*K1*y(n-1) - y(n-2)

	RLC24	DPX1			; adjust decimal point
;
; update past samples with newly computed values
;
	MOVPF16	DPX2,SinCos		; y(n) = 2*K1*y(n-1) - y(n-2)
	MOV16	SinCos_1,SinCos_2	; y(n-2) = y(n-1)
	MOVPF16	DPX2,SinCos_1		; y(n-1) = y(n)
;
; Generation Of The Next Sample Of The Resonator (sine wave) is done.
; The 16 bit result is stored in location "SinCos" (low Byte) &
; "SinCos+1" (High Byte)
;
; write back all the computed values to respective tone variables
;
	decf	fsr0
	bcf	_fs1
	bcf	_fs0
	movfp	SinCos+B1,indf0
	movfp	SinCos+B0,indf0
	movfp	SinCos_1+B1,indf0
	movfp	SinCos_1+B0,indf0
	movfp	SinCos_2C+B2,indf0
	movfp	SinCos_2C+B1,indf0
;
	return
;
;*****************************************************************
;	Include Double Precision Multiplication Routine
;*****************************************************************
SIGNED	equ	TRUE

	include	"17c42mpy.mac"

;
	END
	

