	TITLE   "Digital IIR Filter Using PIC17C42"

	LIST    columns=120,WRAP
;
		#include "17c42.h"
;
		#include "17c42.mac"
;
		#include "17c42iir.mac"
;
;*******************************************************************
;               Second Order Direct Form IIR Filter
;
;
; In the code given below, a 4th order IIR Elliptic Lowpass Filter
; is implemented. Other order filters may be implemented by
; taking the following example code as a basis.
;
;       The specifications of the filter are :
;
;       Sampling Frequency = 2.0 Khz
;
;       Filter Type = 4th Order Elliptic Lowpass Filter
;
;                               Band1           Band2
;       Lower Band Edge         0.0             600 Hz
;       Upper Band Edge         500 Hz          1 Khz
;       Nominal Gain            1.0             0.0
;       Nominal Ripple          0.01            0.05
;       Maximum Ripple          0.00906         0.04601
;       Ripple in dB            0.07830         -26.75
;
;  The Filter Co-efficients for the above specifications
; of the filter are computed as follows :
;
;      1st Section :
;                       A11 = -0.133331
;                       A12 = 0.167145
;                       B10 = 0.285431
;                       B11 = 0.462921
;                       B12 = 0.285431
;      2nd Section
;                       A21 = 0.147827
;                       A22 = 0.765900
;                       B20 = 0.698273
;                       B21 = 0.499908
;                       B22 = 0.698273
;
;
;       Performance (WORST Case):
;
;                       Cycles    = #of Filter Stages*775 + 16
;                                 = 2*775+16 = 1566 Cycles
;                                       ( 391 uSec)
;                               per each sample. Initialization
;                               time after reset is not counted
;                               Timing measured with B0_EQUALS_B2
;                               set to TRUE (see BIQUAD Macro for
;                               explanation
;
;                       Program Memory :
;                                       = 16+ # of FilterStages * (BIQUAD Memory
;						+ filter co-efficients)
;                                               + multiplier
;                                       = 16+2*(63+5)+274 = 421 locations
;                                         (excluding initialization)
;
;                       RAM usage = 48 file registers
;                       RAM usage/each additional stage = 16 file regs
;                       
;                                      
;                          This time is less than 2 Khz (500 uSec),
;                          which means real time filtering is possible
;
;*******************************************************************

;*******************************************************************
;
;
	CBLOCK  0
		B0,B1,B2,B3
	ENDC
;*******************************************************************
;
    CBLOCK  0x18
	DPX,DPX1,DPX2,DPX3              ; arithmetic accumulator
	AARG,AARG1,BARG,BARG1           ; multiply arguments
    ENDC
;
	CBLOCK
		Dn1, Dn1_Hi
		Dn1_1, Dn1_1_Hi
		Dn1_2, Dn1_2_Hi

		Dn2, Dn2_Hi
		Dn2_1, Dn2_1_Hi
		Dn2_2, Dn2_2_Hi
	ENDC

	CBLOCK
		A11, A11_Hi
		A12, A12_Hi
		B10, B10_Hi
		B11, B11_Hi             ; 1st Section Filter Co-efficients
		B12, B12_Hi

		A21, A21_Hi
		A22, A22_Hi
		B20, B20_Hi
		B21, B21_Hi             ; 2nd Section Filter Co-efficients
		B22, B22_Hi
	ENDC

	CBLOCK
		X, X1                   ; 16 bits of input stream
		Y, Y1                   ; 16 bits of filter output

		ACC, ACC1, ACC2, ACC3   ; 32 bit accumulator for computations
	ENDC
;
FltStage        .set     2
NumCoeff        equ     (5*FltStage)    ; 5 Co-eff per stage
;
LPASS   .set    TRUE
HPASS   .set    FALSE
BPASS   .set    FALSE                   ; select the desired filter type
BSTOP   .set    FALSE
;
SIGNED  equ     TRUE                    ; Set This To 'TRUE' for signed multiplication
;                                       ; and 'FALSE' for unsigned.
;
;*******************************************************************
;               Test Program For Low Pass Filter
;*******************************************************************

	ORG     0x0000
;
start
	call    initFilter
	movlw   0x00
	movwf   X
	movlw   0x7f                    ; set initial Xn = X(0) = 0x7f00
	movwf   X+B1                    ; test for impulse response
;
NextPoint
	call    IIR_Filter
	tlwt      _LOW,Y
tracePoint
	tablwt     _HIGH,0,Y+B1
	nop
	clrf     X                      ; set X(n) = 0 , n <> 0
	clrf     X+B1                   ; for simulating an Impulse
	goto    NextPoint
;
self    goto    self
;
;*******************************************************************
;
initFilter
;
;   At first read the Filter Co-efficients from Prog. Mem to Data RAM
;
	#if     LPASS
		movlw    _coeff_lpass
		movwf    tblptrl
		movlw    page    _coeff_lpass
		movwf    tblptrh
	#endif
	#if     HPASS
		movlw    _coeff_hpass
		movwf    tblptrl
		movlw    page    _coeff_hpass
		movwf    tblptrh
	#endif
	#if     BPASS
		movlw    _coeff_bpass
		movwf    tblptrl
		movlw    page    _coeff_bpass
		movwf    tblptrh
	#endif
	#if     BSTOP
		movlw    _coeff_bstop
		movwf    tblptrl
		movlw    page    _coeff_bstop
		movwf    tblptrh
	#endif

;
	movlw    A11
	movwf     fsr0
	bsf      _fs0
	bcf      _fs1                   ; auto increment
;
; Read Filter Co-efficients from Program Memory
;
	movlw    NumCoeff
	tablrd     _LOW,_INC,A11        ; garbage
NextCoeff
	tlrd      _LOW,indf0
	tablrd     _HIGH,_INC,indf0
	decfsz    wreg
	goto    NextCoeff
;
; Initilize "Dn"s to zero
;
	movlw    Dn1
	movwf    fsr0
	movlw    6*FltStage
NextClr
	clrf    indf0
	decfsz  wreg
	goto    NextClr
;
	return
;
;*******************************************************************
;                     1st Cascade Section
;*******************************************************************
;
IIR_Filter
;
; Compute  D(n) = X(n) + A1*D(n-1) + A2*D(n-2)
;          Since the filter constants are computated in Q15 format,
;          X(n) must be multiplied by 2**15 and then added to the
;          other terms.
;
; Move Input to accumulator after proper scaling
;
	bcf     _carry
	rrcf    X+B1
	rrcf    X
	clrf    wreg                    ; Scale the input X
	rrcf    wreg
	movwf   ACC+B1
	movfp   X,wreg
	movwf   ACC+B2
	movfp   X+B1,wreg
	movwf   ACC+B3                  ; ACC = scaled input : X*(2**15)
;
; 1st Biquad filter section
;
	BIQUAD  A11,A12,B10,B11,Dn1,Dn1_1,Dn1_2,1
;
; 2nd Biquad filter section
;
	BIQUAD  A21,A22,B20,B21,Dn2,Dn2_1,Dn2_2,2
;
; The filter output is now computed
; Save the Upper 16 Bits of 32 bit Accumulator into Y after
; adjusting the decimal point
;

	MOV16   ACC+B2,Y
;
	return                          ; Output Y(n) computed
;
;*******************************************************************
;  Set SIGNED/UNSIGNED Flag Before Including 17c42MPY.mac
;
		#include	"17c42MPY.mac"

;*******************************************************************
;  Low Pass Filter Co-efficients
;
;
;                       ELLIPTIC     LOWPASS FILTER
;
;                FILTER ORDER =    4
;                Sampling frequency =   2.000  KiloHertz
;
;                        BAND  1        BAND  2
;
;
; LOWER BAND EDGE         .00000         .60000
; UPPER BAND EDGE         .50000        1.00000
; NOMINAL GAIN           1.00000         .00000
; NOMINAL RIPPLE          .01000         .05000
; MAXIMUM RIPPLE          .00906         .04601
; RIPPLE IN dB            .07830      -26.74235
;
;  I        A(I,1)        A(I,2)        B(I,0)        B(I,1)        B(I,2)
;
;  1      -.133331       .167145       .285431       .462921       .285431
;  2       .147827       .765900       .698273       .499908       .698273
;

_coeff_lpass            ; co-efficients for 1st Cascade section
	data     4369           ; -A11
	data     -5477          ; -A12
	data     9353           ; B10
	data     15169          ; B11
	data     9353           ; B12
;                       ; co-efficients for 2nd Cascade section
	data    -4844           ; -A21
	data    -25097          ; -A22
	data     22881          ; B20
	data     16381          ; B21
	data     22881          ; B22
;
;*******************************************************************
;
;
;*******************************************************************
;  High Pass Filter Co-efficients
;
;
;                       ELLIPTIC    HIGHPASS FILTER
;
;                FILTER ORDER =    4
;                Sampling frequency =   2.000  KiloHertz
;
;                        BAND  1        BAND  2
;
;
; LOWER BAND EDGE         .00000         .50000
; UPPER BAND EDGE         .40000        1.00000
; NOMINAL GAIN            .00000        1.00000
; NOMINAL RIPPLE          .04000         .02000
; MAXIMUM RIPPLE          .03368         .01686
; RIPPLE IN dB         -29.45335         .14526
;
;  I        A(I,1)        A(I,2)        B(I,0)        B(I,1)        B(I,2)
;
;  1       .276886       .195648       .253677      -.411407       .253677
;  2      -.094299       .780396       .678650      -.485840       .678650
;
;
_coeff_hpass                  ; co-efficients for 1st Cascade section
	data     -9073          ; -A11
	data     -6411          ; -A12
	data     8313           ; B10
	data     -13481         ; B11
	data     8313          ; B12
;                       ; co-efficients for 2nd Cascade section
	data    3090            ; -A21
	data    -25572          ; -A22
	data    22238           ; B20
	data    -15920          ; B21
	data    22238           ; B22
;
;*******************************************************************
;
;*******************************************************************
;  Band Pass Filter Co-efficients
;
;
;                       ELLIPTIC     BANDPASS FILTER
;
;                FILTER ORDER =    4
;                Sampling frequency =   2.000  KiloHertz
;
;                        BAND  1        BAND  2        BAND  3
;
;
; LOWER BAND EDGE         .00000         .30000         .90000
; UPPER BAND EDGE         .10000         .70000        1.00000
; NOMINAL GAIN            .00000        1.00000         .00000
; NOMINAL RIPPLE          .05000         .05000         .05000
; MAXIMUM RIPPLE          .03644         .03867         .03641
; RIPPLE IN dB         -28.76779         .32956      -28.77647
;
;
;  I        A(I,1)        A(I,2)        B(I,0)        B(I,1)        B(I,2)
;
;  1      -.936676       .550568       .444000      -.865173       .444000
;  2       .936707       .550568       .615540      1.199402       .615540
;

_coeff_bpass                  ; co-efficients for 1st Cascade section
	data     30693/2           ; -A11
	data     -18041/2          ; -A12
	data     14549/2         ; B10
	data     -28350/2          ; B11
	data     14549/2           ; B12
;                       ; co-efficients for 2nd Cascade section
	data    -30694/2          ; -A21
	data    -18041/2          ; -A22
	data     20170/2          ; B20
	data     39302/2          ; B21
	data     20170/2          ; B22
;
;*******************************************************************
;
;*******************************************************************
;  Band Stop Filter Co-efficients
;
;
;                      ELLIPTIC    BANDSTOP FILTER
;
;               FILTER ORDER =    4
;               Sampling frequency =   2.000  KiloHertz
;
;                       BAND  1        BAND  2        BAND  3
;
;
;LOWER BAND EDGE         .00000         .45000         .70000
;UPPER BAND EDGE         .30000         .55000        1.00000
;NOMINAL GAIN           1.00000         .00000        1.00000
;NOMINAL RIPPLE          .05000         .05000         .05000
;MAXIMUM RIPPLE          .03516         .03241         .03517
;RIPPLE IN dB            .30015      -29.78523         .30027
;
;
;  I        A(I,1)        A(I,2)        B(I,0)        B(I,1)        B(I,2)
;
;  1       .749420       .583282       .392685       .087936       .392685
;  2      -.749390       .583282      1.210022      -.270935      1.210022
;
_coeff_bstop                  ; co-efficients for 1st Cascade section
	data     -24557/2       ; -A11
	data     -19113/2       ; -A12
	data     12868/2        ; B10
	data     2881/2         ; B11
	data     12868/2        ; B12
		       ; co-efficients for 2nd Cascade section
	data    24556/2         ; -A21
	data    -19113/2        ; -A22
	data    39650/2         ; B20
	data    -8878/2         ; B21
	data    39650/2         ; B22
;
;*******************************************************************

	END





