        TITLE   "PULSE WIDTH MODULATION 10 BIT RESOLUTION"
        PROCESSOR      PIC17C42

        
#include        "17c42.h"
       
PWM_HI  equ     0x21 
PWM_LO  equ     0x20
TEMP    equ     0x22
;The user would generate a 16 bit value which is saved in ram 
;locations PWM_HI and PWM_LO byte. In 10 bit mode, the program
;transfers these values directly to the Duty Cycle (DC) registers, to
;generate the required 10 bit PWM.
        NOLIST
;Once the PWM is started, interrupts related to the PWM should be
;disabled. If the PWM output has to be changed do the following:
;In most instances the period is kept constant and the Duty Cycle (DC)
;is varied to get a pwm output. There are some instance in which the
;the period and the duty cycle are changed. Since the DC reg. is double
;buffered but the PRX register is not, the DC registers should be loaded,
;then the "PWM interrupt" should be enabled. When the interrupt does
;occur, the DC slave registers will be loaded with the new DC values
;and the user should now load the new period in the PR1(or 2) register.
;This will give us a "glitch free" PWM output.
        LIST
;
;
;This is a short program to demonstrate how to generate PWM outputs with
;10 bit resolution. Since a 10Mhz crystal was used in the test system.
;The max. period = 1024x100nS = 102.4 uS or 9.8 KHz. This program
;keeps the period constant and varies the duty cycle (which corresponds
;to the most significant 10 bits of the 16 bit value PWM_LO&PWM_HI).
;This program is interrupt driven, i.e. the update to the Duty cycle
;is done in the rtcc interrupt, which then enables the pwm interrupt.
;The period update is done during the pwm interrupt. The pwm output
;ramps up from 0% to 100% duty cycle and then repeats. The full 
;sweep takes approx. 52 secs.
;
;
        ORG     0
        goto    start
;        
        ORG     0x10            ;vector for rtcc interrupt
rtcc_int
        goto    service_rtcc    ;service rtcc
;
        ORG     0x0020          ;vector for pwm interrupt
pwm_int
        goto    service_pwm     ;service pwm only
;
        ORG     0x0030
;
;initialize internal hardware to generate the output
;for 10 bit resolution pwm.
init_pwm10
        movlb   2               
        clrf    tmr1            ;clear timer 1
                                ;used to "drive" pwm1
        setf    pr1             ;set period=9.8 khz
        movlb   3               
        movfp   PWM_HI,pw1dch   ;load duty cyl. hi byte
        movfp   PWM_LO,pw1dcl   ;load duty cycle lo byte
        clrf    tcon1           ;tmr1 inc. internally
                                ;as 8 bit counter
        movlw   00010001B       ;start tmr1 and
        movpf   wreg,tcon2      ;enable pwm1
        movlb   1               
        clrf    pie             ;clr all int. enables
        clrf    pir             ;clear all interrupts
        bsf     _peie           ;except peripheral int.
        retfie
;
;
;everytime a new value is written to the PWM_HI, PWM_LO regs, the
;tmr1 interrupts is enabled. The DC value are written just before
;the "pwm interrupt" is enabled. Here the new period register is
;updated. In this example, period is kept constant at 0xff Tcyl.
service_pwm
        ;if the period changed, write new value here.
        movlb   2               ;select bank 2
        setf    pr1             ;period <-- 0xff
        movlb   1               ;disable tmr1 int
        bcf     _tm1ie          ;       /
        retfie
;
        PAGE
;This part of the program is basically used to simmulate a change
;which would be used to drive the pwm output.
;
;the rtcc is set up to interrupt every 52 mS.
init_rtcc
        movlw   00100000b       ;set up rtcc timer
        movfp   wreg,rtcsta     ;       /
        clrf    rtccl           ;clear rtcc
        clrf    rtcch           ;       /
        movlw   0x80            ;start pwm at 50%
        movwf   PWM_HI          ;       /
        clrf    PWM_LO          ;       /
        bsf     _rtcie          ;enable rtcc int.
        return
;
;Every rtcc interrupt, the PWM_HI&PWM_LO bytes are incremented by 1.
;Only the 10 most significant bits are incremented. Once the 
;
service_rtcc
        bcf     _rtcir          ;reset int flag
        ;do a pseudo inc of the 10 bit PWM_HI, PWM_LO.
        bcf     _carry          ;clear carry
        movlw   01000000b       ;load lsb for 10 bit
        addwf   PWM_LO,1        ;add to LSB
        btfsc   _carry          ;carry?
        incf    PWM_HI          ;yes then inc PWM_HI
        ;now load the values into the Duty Cycle registers
        movlb   3               ;bank 3
        movfp   PWM_LO,pw1dcl   ;load lo value
        movfp   PWM_HI,pw1dch   ;load hi value
        movlb   1
        bsf     _tm1ie          ;enable tmr1 int
        retfie                  
;        
        PAGE
;
start
        bsf     _glintd         ;disable interrupts
        call    init_rtcc       ;initailize the RTCC tmr
                                ;for test purposes
        call    init_pwm10      ;initialize pwm
loop    goto    loop            ;spin wheels
;

        END
